/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdmsql.h is required in all files using RDMSQL */
#include "rdmsql.h"

/* EXAMPLE - SQL10 for C using RDMSQL */
/* This example shows multi-user access to a database.  Two connections are
 * established using shared mode.  In one data is inserted into the database.
 * The other one also opens the database.  The second connection is used to
 * read the data while the first connection still has a transaction open,
 * which fails with a timeout.  Then the transaction is committed (freeing
 * the locks) and the read using the second connection then succeeds.
 *
 * The following RDM functions are illustrated:
 *      rsqlAllocConn/rsqlFreeConn
 *      rsqlAllocStmt/rsqlFreeStmt
 *      rsqlOpenDB/rsqlCloseDB
 *      rsqlExecDirect
 *      rsqlFetch
 *      rsqlTransStart
 *      rsqlTransCommit
 *      rsqlTransRollback
 *      rsqlGetErrorMsg
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 * 
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDMSQL call.
 */
static void drop_database(
    HCONN       hDbc,
    const char *name)
{
    HSTMT hStmt;
    char  commandText[80];

    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Attempt to drop the database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. */
    sprintf(commandText, "DROP DATABASE %s", name);
    (void) rsqlExecDirect(hStmt, commandText);
    (void) rsqlExecDirect(hStmt, "COMMIT");

    (void) rsqlFreeStmt(hStmt);
}

static void create_this_database(
    HCONN       hDbc,
    const char *name,
    const char *cmds[],
    uint32_t    cmd_length)
{
    RSQL_ERRCODE rsqlCode;
    HSTMT        hStmt;
    char         commandText[80];
    uint16_t     ii;

    (void) rsqlAllocStmt(hDbc, &hStmt);

    sprintf(commandText, "CREATE DATABASE %s", name);
    rsqlCode = rsqlExecDirect(hStmt, commandText);

    for (ii = 0; ii < cmd_length; ii++)
    {
        rsqlCode = rsqlExecDirect(hStmt, cmds[ii]);
        if (rsqlCode != errSUCCESS)
            break;
    }

    if (rsqlCode != errSUCCESS)
        fprintf(stderr, "Error creating the %s database\n", name);

    (void) rsqlExecDirect(hStmt, "COMMIT");

    /* Close the newly created database */
    (void) rsqlCloseDB(hDbc, name);

    (void) rsqlFreeStmt(hStmt);
}

static void create_databases(
    HCONN hDbc)
{
    const char *cmds[] = {
        "CREATE TABLE Table1 (id INTEGER PRIMARY KEY)"
    };

    /* Drop any existing copies of the databases. */
    drop_database(hDbc, "SQL10_db");

    /* Attempt to create the databases */
    create_this_database(hDbc, "SQL10_db", cmds, 1);
}

/* =======================================================================
*/
int32_t main(void)
{
    RSQL_ERRCODE      rsqlCode;
    HCONN             hDbc1;
    HCONN             hDbc2;
    HSTMT             hStmt1;
    HSTMT             hStmt2;
    char              commandText[80];
    const RSQL_VALUE *result;
    uint16_t          numCols;
    int16_t           ii;

    /* Create the first connection */
    (void) rsqlAllocConn(&hDbc1);

    create_databases(hDbc1);

    /* Open the database in shared mode */
    rsqlCode = rsqlOpenDB(hDbc1, "SQL10_db", "s");

    /* Create the second connection */
    (void) rsqlAllocConn(&hDbc2);

    /* Open the database in shared mode */
    rsqlCode = rsqlOpenDB(hDbc2, "SQL10_db", "s");

    /* Create a statement in connection 1 and one in connection 2 */
    (void) rsqlAllocStmt(hDbc1, &hStmt1);
    (void) rsqlAllocStmt(hDbc2, &hStmt2);

    /* Start a new transaction */    
    (void) rsqlTransStart(hDbc1, "my_trans");

    /* Insert three rows into the table */
    for (ii = 1; ii < 4; ii++)
    {
        sprintf(commandText, "INSERT INTO Table1 VALUES %hd", ii);
        rsqlCode = rsqlExecDirect(hStmt1, commandText);
    }

    /* Attempt to read the data from connection 2, expect a timeout
     * because connection 1 is holding locks as the transaction is 
     * still active */
    rsqlCode = rsqlExecDirect(hStmt2, "SELECT id FROM Table1");
    if (rsqlCode == errSUCCESS)
    {
        while ((rsqlCode = rsqlFetch(hStmt2, &result, &numCols)) == errSUCCESS)
            printf("Should not get this data - %d\n", result[0].vt.lv);
    }
    else if (rsqlCode == errTIMEOUT)
        printf("Received expected timeout\n");
    else
    {
        const char *errmsg;
        (void) rsqlGetErrorMsg(rsqlCode, &errmsg);
        printf("Unexpected error: %s\n", errmsg);
    }

    /* Commit the transaction */
    (void) rsqlTransCommit(hDbc1);

    rsqlCode = rsqlExecDirect(hStmt2, "SELECT id FROM Table1");

    while ((rsqlCode = rsqlFetch(hStmt2, &result, &numCols)) == errSUCCESS)
        printf("%d\n", result[0].vt.lv);

    /* Close the database */
    (void) rsqlCloseDB(hDbc1, "SQL10_db");

    /* Free handles */
    (void) rsqlFreeStmt(hStmt1);
    (void) rsqlFreeStmt(hStmt2);
    (void) rsqlFreeConn(hDbc1);
    (void) rsqlFreeConn(hDbc2);

    printf("\nPress <Enter> to continue . . .\n");
    getchar();

    return EXIT_SUCCESS;
}

